# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Business Applications
#    Copyright (C) 2012 OpenERP S.A. (<http://openerp.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging
import re
import unicodedata

from openerp.osv import fields, osv
from openerp.tools import ustr
from openerp import SUPERUSER_ID

_logger = logging.getLogger(__name__)

class res_company(osv.Model):
    _inherit = 'res.company'

    def read_anonymous_info(self, cr, uid, context=None):
        '''
        Read the anonymous information and preformat in python to use it to configure the home
        page. Anonymous user will get all the contextual information for the webpage.
        '''
        if context is None: context={}

        ir_model_data = self.pool.get('ir.model.data')
        users_obj = self.pool.get('res.users')
        anonymous_data_id = ir_model_data.get_object_reference(cr, uid, 'portal_anonymous',
                'anonymous_user')
        anonymous_id = ir_model_data.browse(cr, uid, [anonymous_data_id[1]], context=context)
        anonymous_brw = users_obj.browse(cr, uid, [anonymous_id[0].id], context=context)
        a_brw = anonymous_brw and anonymous_brw[0] or {}
        return {
                'company_ids': [a.id for a in a_brw.company_ids], #Only Show this companies in some parts of the page.
                }

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        ir_model_data = self.pool.get('ir.model.data')
        fields_new = fields
        anonymous_data_id = ir_model_data.search(cr, uid, [('model', '=', 'res.users'),
            ('name', '=', 'portal_anonymous.anonymous_user')],
                context=context)
        anonymous_id = ir_model_data.browse(cr, uid, anonymous_data_id, context=context)
        if anonymous_id == uid:
            uid = SUPERUSER_ID
            #Force only secure public fields of my company object.
            #If in portal we overwrite or try to get different fields from the "secure list"
            #You will not recive an access denied you will receive an script error because the rray
            #######will not have the values on it.
            fields_new = [f for f in fields if f in self.secure_fields(cr, uid, ids,
                context=context)]
        return super(res_company, self).read(cr, uid, ids, fields=fields_new,
                context=context,
                load=load)
    def secure_fields(self, cr, uid, context=None):
        return ['name',
                'street',
                'street2',
                'zip',
                'country',
                'phone',
                'fax',
                'email',
                'country_id',
                'type' ]
