openerp.portal_news = function (instance) {
    instance.mail.ThreadComposeMessage.include({
        init: function (parent, datasets, options) {
            if (datasets.options.qweb_compose_message) { this.template = datasets.options.qweb_compose_message; }
            this._super(parent, datasets, options);
        },
    /*Overwrite the init because the template is load wired, rendering the template.
     */
        reinit: function() {
            var $render = $( instance.web.qweb.render(this.template, {'widget': this}) );

            $render.insertAfter(this.$el.last());
            this.$el.remove();
            this.$el = $render;

            this.display_attachments();
            this.bind_events();
        },
    });

    instance.mail.MessageCommon.include({
        /* upload the file on the server, add in the attachments list and reload display
         * If the action bring the attachment special template you can show the attachments in an
         * special way.
         */
        init: function (parent, datasets, options) {
            this._super(parent, datasets, options);
        },
    }); 

    instance.mail.Widget.include({
        init: function (parent, action) {
            if (action.params.qweb_root) { this.template = action.params.qweb_root; }
            this._super(parent, action);
        }
    });

    instance.mail.Wall.include({
        last_search_got: {},
        init: function (parent, action) {
            this._super(parent, action);
            if (action.params.qweb_wall){ this.template = action.params.qweb_wall }
        },
        /**
         * Get the domains, contexts and groupbys in parameter from search
         * view, then render the filtered threads.
         * @param {Array} domains
         * @param {Array} contexts
         * @param {Array} groupbys
         * It is here because it will be overwritten when i understand the logic to pass as 
         * a property the last_search_got
         */
        do_searchview_search: function (domains, contexts, groupbys) {
            var self = this;
            instance.web.pyeval.eval_domains_and_contexts({
                domains: domains || [],
                contexts: contexts || [],
                group_by_seq: groupbys || []
            }).then(function (results) {
                if(self.root) {
                    $('<span class="oe_mail-placeholder"/>').insertAfter(self.root.$el);
                    self.root.destroy();
                }
                return self.message_render(results);
            });
        },
    });

    instance.mail.Thread.include({
        /*
        * As we will use a different template for several ways to show wall
        * We decided overwrite the init to pass the template as parameter.
        * You just need to pass as parameter in the action the template, and load
        * Your template as a qweb file.
        * As a thread is the object that really render all the elements here is where we put all
        * render options.
        */
        init: function (parent, datasets, options) {
            if (options.options.qweb_thread){
                this.template = options.options.qweb_thread;
            }
            this._super(parent, datasets, options);
        },
        /* instantiate the compose message object and insert this on the DOM.
        * The compose message is display in compact form.
        */
        instantiate_compose_message: function () {
            // add message composition form view
            if (!this.compose_message) {
                this.compose_message = new instance.mail.ThreadComposeMessage(this, this, {
                    'context': this.options.compose_as_todo && !this.thread_level ? _.extend(this.context, { 'default_starred': true }) : this.context,
                    'options': this.options,
                });
                if (!this.thread_level || this.thread_level > this.options.display_indented_thread) {
                    this.compose_message.insertBefore(this.$el);
                } else {
                    this.compose_message.prependTo(this.$el);
                }
            }
        },
    });

    instance.mail.ThreadMessage.include({
        init: function(parent, data, options){
            if (data.options.qweb_message) {
                this.template = data.options.qweb_message;
            }
            this._super(parent, data, options);
        },
        /**
         * Bind events in the widget. Each event is slightly described
         * in the function. */
        bind_events: function () {
            var self = this;
            // header icons bindings
            this.$('.oe_read').on('click', this.on_message_read);
            this.$('.oe_unread').on('click', this.on_message_unread);
            this.$('.oe_msg_delete').on('click', this.on_message_delete);
            this.$('.oe_reply').on('click', this.on_message_reply);
            this.$('.oe_star').on('click', this.on_star);
            this.$('.oe_msg_vote').on('click', this.on_vote);
            this.$('.oe_mail_action_form').on('click', this.on_open_message_form);
        },
        on_open_message_form: function(){
            self = this;
            event.stopPropagation();
            this.rpc('/web/action/load', {action_id: 'portal_news.action_view_edit_form'}).done(function(result){
                var ac = _.extend(result || {}, {
                    res_id: self.id
                });
                instance.client.action_manager.do_action(ac);
            });
            $('html, body').animate({ scrollTop: 0 }, 'slow');
        }
    });
    // Especific behavior for the blog and new widgets.
    /*
     * New Widget Footr.
     * TODO:
     * 1.- Extract the Company information from the company.
     * 2.- Customice links.
     * 3.- Absolute Footer.
     */
    instance.portal_news.FooterWeb = instance.web.Widget.extend({
        template: 'FooterWeb',
        init: function(parent){
            self = this;
            this._super(parent);
            //The home page works with the user information setted on anonymous user
            this.companies_obj = new instance.web.DataSet(self, 'res.company', this.ctx);
            this.partners_obj = new instance.web.DataSet(self, 'res.partner', this.ctx);
            this.users_obj = new instance.web.DataSet(self, 'res.users', this.ctx);     
        },
        start: function(){
            this._super.apply(this, arguments);
            this.get_anonymous_info();
        },
        render_all: function(){
            this.wcomp = new instance.portal_news.FooterWebCompanyInfo(this);
            this.wcomp.appendTo(this.$('.oe_placeholder_company_info'));
        },
        get_anonymous_info: function(){
            self = this;
            this.companies_obj.call('read_anonymous_info', {}).done(function(result){
                localStorage.setItem('AnonymousUserInfo', JSON.stringify(result));
                self.render_all();
            });
        }
    });

    instance.portal_news.FooterWebCompanyInfo = instance.web.Widget.extend({
        template: 'FooterWebCompanyInfo',
        init: function(parent){
            this.parent = parent;
            this._super(parent);
        },
        start: function(){
            this._super.apply(this, arguments);
            var self = this;
            this.get_companies_info();
        },
        get_companies_info: function(){
            self = this;
            var companies = JSON.parse(localStorage.getItem('AnonymousUserInfo')).company_ids;
            var ph = this.$('section.oe_address_placeholder');
            self.parent.companies_obj.read_ids(companies,
                ['name', 'street', 'street2', 'country_id', 'fax', 'phone', 'email'])
                .done(function(res){
                _.each(res, function(num){
                    $(instance.web.qweb.render('addressCompany', {'widget': num})).appendTo(ph);
                });
            });
        },
        on_load_company_info: function(result){
        }
    });

    /*
     * End of generic Footer
     */

    /* With this widget we will have a link to see in an normal OpenERP tree view the content
     * it simply call an action with the domain available in the params of the action.
     * param: 
     *  {options}: options in client_action to get the domain from it.
     */
    instance.portal_news.EditContent = instance.web.Widget.extend({
        template: 'EditContent',
        domain: [],
        init: function(parent){
            if (parent.action.params.domain){
                this.domain = parent.action.params.domain;
            }
            this._super(parent);
        },
        start: function () {
            this._super.apply(this, arguments);
            this.bind_events();
        },
        bind_events: function() {
            self = this;
            this.$(".oe_edit_content_gallery").click(function (event) {
                event.stopPropagation();
                self.rpc("/web/action/load", 
                    { action_id: "mail.action_view_mail_message" })
                    .done(function(result) {
                    result.domain = self.domain;
                    //Here TODO: I SHOULD CATCH THE SEARCH ON THE VIEW.
                    // But I dont know where it is.
                    instance.client.action_manager.do_action( result );
                });                                                             
            });
        },
    });
    // Just for Blog Feature.
    /*
     * The blog feature is how we will present the blog to the world.
     * Here is where we inherit the Wall (to have an specific behavior)
     * and attach the footer (and other widgets)
     * Here is where we declarate the client action to have the customized windows.
     */
    instance.portal_news.Blog = instance.mail.Wall.extend({
        start: function(){
            this._super.apply(this, arguments);
        /*
        * Assembly the blog.
        * First the Links to edit.
        * Second put the footer in place.
        */
            //Force bs3 class only in the enviroments rendered as blog, to overwrite with only css
            //the background color and some styles.
            $('.oe_application').addClass('bs3');
            var Links = new instance.portal_news.EditContent(this);
            Links.appendTo(this.$el.find('section.page-header'));
            //TODO: Make the concept of footer generic.
            var Footer = new instance.portal_news.FooterWeb();
            Footer.appendTo(this.$el);
        },
    });
    instance.web.client_actions.add('mail.wall.blog', 'instance.portal_news.Blog');
    /*
     * End of Blog Features
     */

    /*
       This include is to add footer in a view, this requires placing the class "bs3-footer"
        in the form declaration of view.
    */
    instance.web.FormView.include({
            /*
        start: function(){
            self = this
            this._super.apply(this, arguments).done(function () {
                if (typeof self.$(".bs3-footer")[0] != "undefined") {
                    var Footer = new instance.portal_news.FooterWeb();
                    Footer.appendTo(self.$el);
                }
            });
        }
            */
    });

};

